/*
 * TextNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso;


import java.util.List;

import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.StreamObjectVisitor;
import org.jabberstudio.jso.StreamText;

/**
 * <p>
 * Basic implementation of the <code>StreamText</code> interface.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class TextNode extends AbstractNode implements StreamText {
    //Fields
    private StreamElement   _Parent;
    private String          _Val;

    //Constructors
    public TextNode(StreamDataFactory sdf, String val) {
        this (null, sdf, val);
    }
    public TextNode(StreamElement parent, StreamDataFactory sdf, String val) {
        super(sdf);
        setParent(parent);
        setValue(val);
    }
    
    //Methods
    /** <p>
     * Retrieves the parent <code>StreamElement</code> for this
     * <code>StreamNode</code>.</p>
     *
     * <p>
     * The value returned by this method may be <code>null</code>, if the
     * parent has not yet been defined.</p>
     *
     * @return The <code>StreamElement</code> parent.
     */
    public StreamElement getParent() {
        return _Parent;
    }
    /** <p>
     * Sets the parent <code>StreamElement</code> for this
     * <code>StreamNode</code>.</p>
     *
     * @param p the <code>StreamElement</code> parent.
     */
    public void setParent(StreamElement p) {
        if (_Parent != null) {
            _Parent.remove(this);
        }
        _Parent = p;
    }
    
    /** <p>
     * Retreives the String value for this <code>StreamText</code>.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * @return the String for this <code>StreamText</code>.
     */
    public String getValue() {
        return _Val;
    }
    /** <p>
     * Sets the String value for this <code>StreamText</code>.</p>
     *
     * <p>
     * The value of <code>val</code> cannot be <code>null</code> or "",
     * otherwise an <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param val the new String value for this <code>StreamText</code>.
     * @throws IllegalArgumentException If the parameter is invalid.
     */
    public void setValue(String val) throws IllegalArgumentException {
        if (val == null)
            val = "";
        _Val = val;
    }
    
    /** <p>
     * Lists the <code>StreamNode</code> objects that are contained by this
     * <code>StreamNode</code>.  This list may or may not be modifiable.</p>
     *
     * <p>
     * The value returned by this method is never <code>null</code>.</p>
     *
     * @return The list of contained <code>StreamNode</code> objects.
     */
    public List listNodes() {
        return java.util.Collections.EMPTY_LIST;
    }

    /** <p>
     * Creates an "almost-exact" duplicate of this <code>StreamNode</code>,
     * using the given parent <code>StreamElement</code>.  By "almost-exact",
     * all information for this <code>StreamNode</code>, and its contained
     * objects (obtained via <code>{@link #listNodes}</code>) are duplicated.
     * The parent for this <code>StreamNode</code> is one specified by
     * <code>p</code>, rather than thatreferenced by this
     * <code>StreamNode</code>.</p>
     *
     * <p>
     * The value of <code>p</code> may be <code>null</code>, in which case no
     * parent is attached to the newly created <code>StreamNode</code>.</p>
     *
     * @return the "almost-exact" duplicate of this <code>StreamNode</code>.
     */
    public StreamObject copy(StreamElement p) {
        return new TextNode(p, getDataFactory(), getValue());
    }
    
    /**
     * <p>
     * Compares this <code>TextNode</code> to the given object.  This
     * comparison determines if this <code>cmp</code> to this instance of
     * <code>TextNode</code>.</p>
     *
     * @return <code>true</code> if <code>cmp</code> refers to this
     * <code>TextNode</code>.
     */
    public boolean equals(Object cmp) {
        return this == cmp;
    }

    /**
     * <p>
     * Returns a string representation of this <code>TextNode</code>.  This
     * method returns the same results as <code>{@link #getValue}</code>.</p>
     *
     * @return The string representing this <code>TextNode</code>.
     */
    public String toString() {
        return getValue();
    }
    
    /** <p>
     * Accepts a <code>StreamNodeVisitor</code> for processing.  This is the
     * "accepting" method implementing the <b><i>Visitor</i></b> design
     * pattern.</p>
     *
     * @param v The <code>StreamNodeVisitor</code> to accept.
     */
    public void accept(StreamObjectVisitor v) {
        v.visit(this);
    }
}