/*
 * DOMParseState.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.io;


import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.io.StreamParseState;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class DOMParseState implements StreamParseState {
    //Fields
    private DOMParseState   _Parent;
    private Node            _Node;
    private List            _Nodes = new java.util.LinkedList();

    private Type            _Type;
    private StreamElement   _Ctx;
    
    private NSI             _Name;
    private Map             _Nss = new java.util.TreeMap();
    private Map             _Attrs = new java.util.TreeMap(NSI.NS_LOOSE_COMPARATOR);
    
    private String          _Text;

    //Constructors
    /** Creates a new instance of DOMParseState */
    protected DOMParseState(StreamElement ctx, DOMParseState p, Node n) {
        _Ctx = ctx;
        _Parent = p;
        _Node = n;
    }
    
    //Methods
    public static DOMParseState createForElement(StreamElement ctx, DOMParseState parent, Element e) {
        DOMParseState   state = new DOMParseState(ctx, parent, e);
        String          ln = getLocalName(e.getNodeName()), prefix = getPrefix(e.getNodeName());
        String          uri = findNamespaceURI(e, state, ctx);
        NamedNodeMap    attrs = e.getAttributes();
        Node            child;
        int             idx;
        
        //Setup Attributes
        for (idx = 0; idx < attrs.getLength(); idx++) {
            Attr    attr = (Attr)attrs.item(idx);
            String  attrQN = attr.getName();
            String  attrLN = getLocalName(attrQN);
            String  attrPre = getPrefix(attrQN);
            String  attrURI = attr.getNamespaceURI();
            String  attrVal = attr.getValue();

            if          (attrQN.equals("xmlns")) {
                //Declares Default Namespace
                state.addNamespace(attrVal, "");
                uri = attrVal;
            } else if   (attrQN.startsWith("xmlns:")) {
                //Declares Prefixed Namespace
                state.addNamespace(attrVal, attrLN);
                if (attrLN.equals(prefix))
                    uri = attrVal;
            } else {
                //Plain Old Attribute
                state.setAttributeValue(attrLN, attrURI, attrVal);
            }
        }
        
        //Setup children
        for (child = e.getFirstChild(); child != null; child = child.getNextSibling()) {
            if ((child instanceof Element) || (child instanceof Text)) {
                state.addNode(child);
            }
        }
        
        //Setup name
        state.setName(ln, uri);
        
        return state;
    }
    public static DOMParseState createForText(StreamElement ctx, DOMParseState parent, Text t) {
        DOMParseState   state = new DOMParseState(ctx, parent, t);
        
        //Setup text
        state.setText(t.getNodeValue());
        
        //Setup type
        state.setType(TEXT);
        
        return state;
    }
    
    private static String getLocalName(String qname) {
        String  local;
        int     pos = qname.indexOf(":");
        
        if (pos != -1) {
            local = qname.substring(pos + 1);
        } else {
            local = qname;
        }
        
        return local;
    }
    private static String getPrefix(String qname) {
        String  prefix;
        int     pos = qname.indexOf(":");
        
        if (pos != -1) {
            prefix = qname.substring(0, pos);
        } else {
            prefix = null;
        }
        
        return prefix;
    }
    private static String findNamespaceURI(Element e, DOMParseState parent, StreamElement ctx) {
        String  uri = e.getNamespaceURI();
        String  prefix = getPrefix(e.getNodeName());
        
        if ((uri == null) && (parent != null))
            uri = parent.getNamespaceURI(prefix);
        if ((uri == null) && (ctx != null))
            uri = ctx.getNamespaceURI(prefix);
        
        return uri;
    }
    
    public DOMParseState getParent() {
        return _Parent;
    }
    
    /** <p>
     * Retrieves the <tt>Type</tt> for this <tt>StreamState</tt>.
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>Type</tt> for this <tt>StreamState</tt>.
     */
    public Type getType() {
        return _Type;
    }
    public void setType(Type t) throws IllegalArgumentException {
        if (t == null)
            throw new IllegalArgumentException("Type cannot be null");
        
        _Type = t;
    }
    /** <p>
     * Retrieves the <tt>StreamContext</tt> for this
     * <tt>StreamState</tt>.  The <tt>StreamContext</tt> describes the
     * "direction" of the data, and provides such things as the appropriate
     * <tt>StreamDataFactory</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamContext</tt> for this <tt>StreamState</tt>.
     */
    public StreamElement getContext() {
        return _Ctx;
    }
    /** <p>
     * Retrieves the current depth for this <tt>StreamState</tt>.  The
     * depth is based on the current "element tree", using the root tag
     * (the &lt;stream:stream&gt;) for position 1.  Matching start and end tags
     * have the same depth.</p>
     *
     * <p>
     * The value returned by this method is never less than 0.</p>
     *
     * @return The current tag depth.
     */
    public int getDepth() {
        return (getParent() != null) ? getParent().getDepth() + 1 : 1;
    }
    /** <p>
     * Retrieves the "raw" XML string for the current <tt>StreamState</tt>.
     * This string is exactly what the XML processor has just processed.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if the
     * underlying XML parser/processor does not provide this information.</p>
     *
     * @return The "raw" XML string.
     */
    public String getXML() {
        return _Node.toString();
    }

    /** <p>
     * Retrieves the element name for this <tt>StreamState</tt>.  This
     * contains the local name and namespace URI for the element (either
     * start tag or end tag).</p>
     *
     * <p>
     * The value returned by this method is not <tt>null</tt> only if
     * {@link #getType} returns {@link #ELEMENT_START} or {@link
     * #ELEMENT_END}.</p>
     *
     * @return The name (local name and namespace URI) for this
     * <tt>StreamState</tt>
     */
    public NSI getName() {
        return _Name;
    }
    private void setName(NSI name) {
        _Name = name;
    }
    private void setName(String localname, String uri) {
        setName(new NSI(localname, uri));
    }
    
    protected Map getNamespaces() {
        return _Nss;
    }
    private void addNamespace(String uri, String prefix) {
        if (uri == null)
            uri = "";
        if (prefix == null)
            prefix = "";
        
        _Nss.put(prefix, uri);
    }
    /** <p>
     * Retrieves the set of namespace URI's for the given depth.  The returned
     * set only contains URI's declared at the given depth.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @param depth The depth to retrieve for.
     * @return The set of declared Namespace URI's
     */
    public Set getNamespaceURIs(int depth) {
        if ((depth < getDepth()) && (getParent() != null))
            return getParent().getNamespaceURIs(depth);
        return new java.util.TreeSet(getNamespaces().values());
    }
    public Set getNamespacePrefixes(int depth) {
        if ((depth < getDepth()) && (getParent() != null))
            return getParent().getNamespacePrefixes(depth);
        return getNamespaces().keySet();
    }
    /** <p>
     * Retrieves the namespace prefix for the given URI (if found).  This
     * method searches all depths, starting with the current depth, for
     * the first match to <tt>uri</tt>.</p>
     *
     * <p>
     * The value of <tt>uri</tt> can be <tt>null</tt> or "", in which
     * case it is considered to be "" (or the "non" namespace).</p>
     *
     * <p>
     * This method returns <tt>null</tt> if a match for <tt>uri</tt>
     * cannot be found.  If <tt>uri</tt> is the default namespace, this
     * method returns "".</p>
     *
     * @param uri The namespace URI to search for.
     * @return The prefix matching the given URI.
     */
    public String getNamespacePrefix(String uri) {
        String      prefix = null;
        Iterator    itr = getNamespaces().values().iterator();
        
        if (uri == null)
            uri = "";
        
        while ((prefix == null) && itr.hasNext()) {
            String      temp = (String)itr.next();
            
            if (temp.equals(uri))
                prefix = temp;
        }
        
        if ((prefix == null) && (getParent() != null))
            prefix = getParent().getNamespacePrefix(uri);
        
        return prefix;
    }
    /** <p>
     * Retrieves the namespace URI for the given prefix (if found).  This
     * method searches all depths, starting with the current depth, for the
     * first match to <tt>prefix</tt>.</p>
     *
     * <p>
     * The value of <tt>prefix</tt> can be <tt>null</tt> or "", in
     * which case it is considered to be "" (or the default namespace).</p>
     *
     * <p>
     * This method returns <tt>null</tt> if a match for <tt>prefix</tt>
     * cannot be found.  If <tt>prefix</tt> is for the "non" namespace, this
     * method returns "".</p>
     *
     * @param prefix The namespace prefix to search for.
     * @return The URI matching the given prefix.
     */
    public String getNamespaceURI(String prefix) {
        String      uri;

        if (prefix == null)
            prefix = "";
        
        uri = (String)getNamespaces().get(prefix);
        if ((uri == null) && (getParent() != null))
            uri = getParent().getNamespaceURI(prefix);
        
        return uri;
    }

    protected Map getAttributes() {
        return _Attrs;
    }
    /** <p>
     * Retrieves the set of attribute names (as <tt>NSI</tt> objects).
     * The returned set does not include any namespace declarations.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt> if the
     * value of {@link #getType} is {@link #ELEMENT_START} or {@link
     * #ELEMENT_END}.</p>
     *
     * @return The set of declared attributes.
     */
    public Set getAttributeNames() {
        return getAttributes().keySet();
    }
    /** <p>
     * Retrieves the attribute value with the given (local) name.  This method
     * returns the value of the first attribute located, regardless of
     * namespace.  If more than two attributes have the same local name (but
     * different namespaces), the one returned is not defined by position
     * in XML.  Note that this method always returns <tt>null</tt> if the
     * value returned by {@link #getType} is not {@link #ELEMENT_START} or
     * {@link #ELEMENT_END}.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no
     * attribute with the given name exists.</p>
     *
     * @param name The (local) name of the attribute.
     * @return The value of the attribute.
     * @throws IllegalArgumentException If the parameter is invalid.
     */
    public String getAttributeValue(String name) throws IllegalArgumentException {
        return getAttributeValue(new NSI(name, null));
    }
    /** <p>
     * Retrieves the attribute value with the given name.  This method
     * returns the value of the attribute matching both the "local name" and
     * namespace URI of <tt>name</tt>.  If the namespace URI for
     * <tt>name</tt> is <tt>null</tt>, the bahavior of this method is
     * the same as {@link #getAttributeValue(String)}.  Note that this method
     * always returns <tt>null</tt> if the value returned by {@link
     * #getType} is {@link #ELEMENT_START} or {@link #ELEMENT_END}.</p>
     *
     * <p>
     * The value of <tt>name</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.  Also, the value
     * <tt>name</tt> returns for {@link NSI#getLocalName} cannot be
     * <tt>null</tt> or "", or an <tt>IllegalArgumentException</tt>
     * is thrown.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if no
     * attribute with the given name exists.</p>
     *
     * @param name The full name of the attribute.
     * @return The value of the attribute.
     * @throws IllegalArgumentException If the parameter is invalid.
     */
    public String getAttributeValue(NSI name) throws IllegalArgumentException {
        if (name == null)
            throw new IllegalArgumentException("Name cannot be null");
        
        if (!Utilities.isValidString(name.getLocalName()))
            throw new IllegalArgumentException("Local Name cannot be null or \"\"");
        
        return (String)getAttributes().get(name);
    }
    private void setAttributeValue(String name, String value) throws IllegalArgumentException {
        setAttributeValue(new NSI(name, null), value);
    }
    private void setAttributeValue(String name, String uri, String value) throws IllegalArgumentException {
        setAttributeValue(new NSI(name, uri), value);
    }
    private void setAttributeValue(NSI name, String value) throws IllegalArgumentException {
        if (name == null)
            throw new IllegalArgumentException("Name cannot be null");
        
        if (!Utilities.isValidString(name.getLocalName()))
            throw new IllegalArgumentException("Local Name cannot be null or \"\"");

        if (!getAttributes().containsKey(name) && (name.getNamespaceURI() == null))
            name = new NSI(name.getLocalName(), "");
        
        if (value == null)
            getAttributes().remove(name);
        else
            getAttributes().put(name, value);
    }
    
    public List getNodes() {
        return _Nodes;
    }
    private void addNode(Node n) {
        _Nodes.add(n);
    }
    
    /** <p>
     * Retrieves the text for this <tt>StreamState</tt>.</p>
     *
     * <p>
     * The value returned by this method is not <tt>null</tt> if the
     * value returned by {@link #getType} is {@link #TEXT}.</p>
     *
     * @return The text string for this <tt>StreamState</tt>.
     */
    public String getText() {
        return _Text;
    }
    private void setText(String text) {
        _Text = text;
    }
}