/*
 * SAXReader.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.io;


import java.io.IOException;
import java.io.StringReader;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.NamespaceSupport;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamContext;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNamespace;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.io.XMLImporter;
import org.jabberstudio.jso.util.Utilities;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class XMLReader implements XMLImporter {
    //Inner-classes
    protected class BuildPlace {
        //Fields
        public final StreamBuilder  builder;
        public final XMLParseState  parseState;
        
        //Constructors
        public BuildPlace(XMLParseState ps, StreamBuilder builder) {
            this.parseState = ps;
            this.builder = builder;
        }
    }
    
    protected class SAXHandler extends DefaultHandler {
        //Fields
        private NamespaceSupport	_Nss = new NamespaceSupport();
        
        //Methods
        protected String getPrefix(String qn) {
            String      prefix = "";
            int         pos = qn.indexOf(':');
            
            if (pos != -1)
                prefix = qn.substring(0, pos);
            
            return prefix;
        }
        
        public void startDocument() {
            StreamElement       ctx = getContext();
            Iterator            itr = ctx.getDeclaredNamespaces().iterator();
            StreamNamespace     ns;
            
            while (itr.hasNext()) {
                ns = (StreamNamespace)itr.next();
                
                _Nss.declarePrefix(ns.getPrefix(), ns.getURI());
            }
            _Nss.pushContext();
        }
        
        public void startElement(String uri, String ln, String qn, Attributes atts) throws SAXException {
            BuildPlace              bp = topPlace();
            StreamBuilder           builder;
            StreamBuilder.Status    stat;
            XMLParseState           current, parent;
            NSI                     name, attrName;
            Map                     nss = new java.util.TreeMap(), attrs = new java.util.TreeMap(NSI.STRICT_COMPARATOR);
            String                  prefix, attrLN, attrURI, attrVal;
            String                  nsPrefix, nsURI;
            int                     idx;
            Enumeration             itr;
            
            //Setup previous ParseState
            parent = ((bp != null) ? bp.parseState : null);
            
            //Setup name
            prefix = getPrefix(qn);
            if (!Utilities.isValidString(uri))
                uri = (String)_Nss.getURI(prefix);
            if (!Utilities.isValidString(uri))
                uri = getCurrentElement().getNamespaceURI(prefix);
            if (!Utilities.isValidString(uri))
                uri = null;
            
            name = new NSI(ln ,uri);
            
            //Setup namespaces
            itr = _Nss.getDeclaredPrefixes();
            while (itr.hasMoreElements()) {
                nsPrefix = (String)itr.nextElement();
                nsURI = _Nss.getURI(nsPrefix);
                
                nss.put(nsPrefix, nsURI);
            }
            
            //Setup attributes
            for (idx = 0; idx < atts.getLength(); idx++) {
                attrLN = atts.getLocalName(idx);
                attrURI = atts.getURI(idx);
                attrVal = atts.getValue(idx);
                attrName = new NSI(attrLN, attrURI);

                attrs.put(attrName, attrVal);
            }
            
            //Setup current ParseState and StreamBuilder
            current = XMLParseState.createForElementStart(getContext(), parent, name, nss, attrs);
            _Nss.pushContext();
            
            builder = setupBuilder(current, (bp != null) ? bp.builder : null);
            while ((stat = builder.processElementStart(current)) == StreamBuilder.EXTENDED_DATA) {
                builder = builder.buildExtendedData();
            }
            pushPlace(new BuildPlace(current, builder));
        }
        public void endElement(String uri, String ln, String qn) throws SAXException {
            BuildPlace              currBP = popPlace(), prevBP = topPlace();
            StreamBuilder           builder;
            StreamBuilder.Status    stat;
            XMLParseState           current, previous;
            StreamElement           currElem, prevElem;
            NSI                     name;
            
            if (currBP != null) {
                name = currBP.parseState.getName();
                builder = currBP.builder;
                previous = ((prevBP != null) ? prevBP.parseState : null);
                current = XMLParseState.createForElementEnd(getContext(), previous, name);
                stat = builder.processElementEnd(current);
                
                if (stat == StreamBuilder.COMPLETE) {
                    currElem = (StreamElement)builder.getNode(); 
                    
                    if (prevBP != null)
                        prevBP.builder.addExtendedData(currElem);
                    setOutput(currElem);
                }
                
                _Nss.popContext();
            }
        }
        public void characters(char[] ch, int start, int length) throws SAXException {
            BuildPlace              currBP = topPlace();
            XMLParseState           current, previous;
            StreamBuilder           builder;
            StreamBuilder.Status    stat;
            String                  txt;
            
            if (currBP != null) {
                txt = new String(ch, start, length);
                builder = currBP.builder;
                previous = currBP.parseState;
                current = XMLParseState.createForText(getContext(), previous, txt);
                stat = builder.processText(current);
            }
        }
        
        public void startPrefixMapping(String prefix, String uri) throws SAXException {
            _Nss.declarePrefix(prefix, uri);
        }
        public void endPrefixMapping(String prefix) throws SAXException {
        }
    }
    
    //Fields
    private StreamElement   _Ctx;
    private StreamElement   _Out;
    
    private List            _States = new java.util.LinkedList();
    
    //Constructors
    public XMLReader() {
    }
    public XMLReader(StreamElement ctx) throws IllegalArgumentException {
        setContext(ctx);
    }
    
    //Methods
    public StreamElement getContext() {
        return _Ctx;
    }
    private void setContext(StreamElement ctx) throws IllegalArgumentException {
        if (ctx == null)
            throw new IllegalArgumentException("context StreamElement cannot be null");
        _Ctx = ctx;
    }
    
    public void configure(StreamElement ctx) throws IllegalArgumentException, IllegalStateException, RuntimeException {
        if (_Ctx != null)
            throw new IllegalStateException("Importer has already been initialized");
        
        _Ctx = ctx;
    }
    
    public StreamElement read(String in) throws RuntimeException {
        StreamElement       out = null;
        SAXParserFactory    factory;
        SAXParser           parser;
        
        try {
            factory = SAXParserFactory.newInstance();
            factory.setNamespaceAware(true);
            factory.setValidating(false);

            parser = factory.newSAXParser();
            parser.parse(new InputSource(new StringReader(in)), new SAXHandler());
            out = getOutput();
        } catch (IOException ioe) {
            throw new RuntimeException("unknown I/O exception occured", ioe);
        } catch (ParserConfigurationException pce) {
            throw new RuntimeException("could not create or configure parser", pce);
        } catch (SAXParseException saxpe) {
            throw new RuntimeException("could not parse input", saxpe);
        } catch (SAXException saxe) {
            throw new RuntimeException("unknown SAX exception occured", saxe);
        }
        
        return out;
    }
    public StreamElement execute(Object in) throws RuntimeException {
        return read((in != null) ? in.toString() : "");
    }
    
    protected StreamBuilder setupBuilder(XMLParseState current, StreamBuilder b) {
        StreamElement       ctx;
        StreamDataFactory   sdf;
        NSI                 name;
        
        if (b == null) {
            ctx = getContext();
            name = current.getName();
            if (name.equals(ctx.getNSI())) {
                b = ctx.createBuilder();
            } else {
                b = ctx.getDataFactory().createElementBuilder(name, null);
            }
        }
        
        return b;
    }
    
    protected StreamElement getOutput() {
        return _Out;
    }
    protected void setOutput(StreamElement out) {
        _Out = out;
    }
    
    protected StreamElement getCurrentElement() {
        StreamElement       current = getOutput();
        
        return ((current == null) ? getContext() : current);
    }
    
    protected BuildPlace topPlace() {
        return (BuildPlace)(!_States.isEmpty() ? _States.get(0) : null);
    }
    protected void pushPlace(BuildPlace bp) {
        _States.add(0, bp);
    }
    protected BuildPlace popPlace() {
        return (BuildPlace)(!_States.isEmpty() ? _States.remove(0) : null);
    }
}