/*
 * SIQueryNode.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.outer_planes.jso.x.si;


import java.util.List;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamDataFactory;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.StreamNode;
import org.jabberstudio.jso.StreamObject;
import org.jabberstudio.jso.io.StreamBuilder;
import org.jabberstudio.jso.io.StreamParseState;
import org.jabberstudio.jso.util.Utilities;
import org.jabberstudio.jso.x.fneg.FeatureNegQuery;
import org.jabberstudio.jso.x.si.*;

import net.outer_planes.jso.ExtensionBuilder;
import net.outer_planes.jso.ExtensionNode;

/**
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class SIQueryNode extends ExtensionNode implements SIQuery {
    //"Constants"
    public static final NSI     ATTRNAME_MIME       = new NSI("mime-type", null);
    public static final NSI     ATTRNAME_PROFILE    = new NSI("profile", null);
    
    public static final String  FEATURE_METHOD      = "stream-method";
    
    //Constructors
    public SIQueryNode(StreamDataFactory sdf) {
        super(sdf, NAME);
    }
    protected SIQueryNode(StreamElement parent, SIQueryNode base) {
        super(parent, base);
    }
    
    //Methods
    public StreamBuilder createBuilder() {
        return new ExtensionBuilder(this) {
            protected StreamBuilder.Status processNestedElementStart(StreamParseState s) {
                StreamDataFactory       sdf     = s.getContext().getDataFactory();
                NSI                     name    = s.getName();
                StreamBuilder.Status    stat    = EXTENDED_DATA;
                String                  profile = getAttributeValue(ATTRNAME_PROFILE);
                
                if          (NSI.STRICT_COMPARATOR.compare(name, FeatureNegQuery.NAME) == 0) {
                    setExtendedData(sdf.createElementBuilder(name, FeatureNegQuery.class));
                } else if   (Utilities.isValidString(profile) && Utilities.equateStrings(name.getNamespaceURI(), profile)) {
                    setExtendedData(sdf.createElementBuilder(name, SIProfile.class));
                } else {
                    stat = super.processNestedElementStart(s);
                }
                
                return stat;
            }
        };
    }
    
    public String getMimeType() {
        String      mime = getAttributeValue(ATTRNAME_MIME);
        
        return ((mime != null) ? mime : "");
    }
    public void setMimeType(String mime) {
        if (!Utilities.isValidString(mime))
            mime = null;
        setAttributeValue(ATTRNAME_MIME, mime);
    }
    
    public SIProfile getProfile() {
        String          name = getAttributeValue(ATTRNAME_PROFILE);
        StreamElement   elem = (Utilities.isValidString(name) ? getFirstElement(null, name) : null);
        SIProfile       prof = null;
        
        if (elem != null) {
            if (elem instanceof SIProfile) {
                prof = (SIProfile)elem;
            } else {
                prof = (SIProfile)getDataFactory().createElementNode(elem.getNSI(), SIProfile.class);
                
                prof.reset(elem);
                insert(indexOf(elem), prof);
                elem.detach();
            }
        }
        
        return prof;
    }
    public void setProfile(SIProfile prof) {
        clearElements(SIProfile.class);
        add(prof);
    }
    public SIProfile applyProfile(NSI name) throws IllegalArgumentException {
        return applyProfile(name, null);
    }
    public SIProfile applyProfile(NSI name, Class iface) throws IllegalArgumentException {
        SIProfile       profile;

        //Validate name
        if (name == null)
            throw new IllegalArgumentException("Name cannot be null");
        if (!Utilities.isValidString(name.getLocalName()))
            throw new IllegalArgumentException("Local-name cannot be null or \"\"");
        if (!Utilities.isValidString(name.getNamespaceURI()))
            throw new IllegalArgumentException("Namespace URI cannot be null or \"\"");
        
        //Validate interface
        if          (iface == null) {
            iface = SIProfile.class;
        } else if   (!SIProfile.class.isAssignableFrom(iface)) {
            throw new IllegalArgumentException("Interface must be null or a superclass of SIProfile");
        }
        
        profile = (SIProfile)addElement(name, iface);
        
        return profile;
    }
    
    public FeatureNegQuery getStreamMethodNegotiation() {
        FeatureNegQuery     fneg = (FeatureNegQuery)getFirstElement(FeatureNegQuery.class);
        
        if (fneg == null) {
            fneg = (FeatureNegQuery)addElement(FeatureNegQuery.NAME, FeatureNegQuery.class);
            
            fneg.putFeature(FEATURE_METHOD, null);
        }
        
        return fneg;
    }
    
    public List listSupportedStreamMethods() {
        return getStreamMethodNegotiation().getSupportedOptions(FEATURE_METHOD);
    }
    public void addSupportedStreamMethod(String method) {
        FeatureNegQuery     fneg = getStreamMethodNegotiation();
        List                supported = fneg.getSupportedOptions(FEATURE_METHOD);
        
        if (Utilities.isValidString(method) && !supported.contains(method)) {
            supported.add(method);
            
            fneg.putFeature(FEATURE_METHOD, supported);
        }
    }
    public void removeSupportedStreamMethod(String method) {
        FeatureNegQuery     fneg = getStreamMethodNegotiation();
        List                supported = fneg.getSupportedOptions(FEATURE_METHOD);
        
        if (Utilities.isValidString(method) && supported.contains(method)) {
            supported.remove(method);
            
            fneg.putFeature(FEATURE_METHOD, supported);
        }
    }
    public void clearSupportedStreamMethods() {
        getStreamMethodNegotiation().putFeature(FEATURE_METHOD, null);
    }
    
    
    public String getPreferredStreamMethod() {
        return getStreamMethodNegotiation().getPreferredOption(FEATURE_METHOD);
    }
    public void setPreferredStreamMethod(String method) {
        getStreamMethodNegotiation().setPreferredOption(FEATURE_METHOD, method);
    }
    
    public void add(StreamNode node) {
        SIProfile       prof;
        
        if (node instanceof SIProfile) {
            prof = (SIProfile)node;
            setAttributeValue(ATTRNAME_PROFILE, prof.getNamespaceURI());
        }
        super.add(node);
    }
    public void insert(int idx, StreamNode node) {
        SIProfile       prof;
        
        if (node instanceof SIProfile) {
            prof = (SIProfile)node;
            setAttributeValue(ATTRNAME_PROFILE, prof.getNamespaceURI());
        }
        super.insert(idx, node);
    }
    public void remove(StreamNode node) {
        SIProfile       prof;
        
        if (node instanceof SIProfile) {
            prof = (SIProfile)node;
            setAttributeValue(ATTRNAME_PROFILE, null);
        }
        super.remove(node);
    }

    public StreamObject copy(StreamElement parent) {
        return new SIQueryNode(parent, this);
    }
}