/*
 * PacketRouter.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso;


/**
 * <p>
 * Interface for routing <tt>Packet</tt> objects. This interface provides for
 * sending and processing <tt>Packet</tt> objects, and checking the current
 * status.</p>
 *
 * <p>
 * This interface is used in conjunction with {@link
 * org.jabberstudio.jso.event.PacketDispatcher}, which handles the receiving
 * of <tt>Packet</tt> objects.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.10
 */
public interface PacketRouter {
    //Methods
    /**
     * <p>
     * Retrieves the <tt>StreamDataFactory</tt> for this <tt>PacketRouter</tt>.
     * This instance should be used to create all <tt>Packet</tt> objects sent
     * to this <tt>PacketRouter</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamDataFactory</tt>.
     * @since   JSO-0.12
     */
    public StreamDataFactory getDataFactory();
    /**
     * <p>
     * Retrieves the <tt>StreamContext</tt> for inbound operations. This
     * <tt>StreamContext</tt> should be referred to within {@link #process}.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamContext</tt> for inbound operations.
     */
    public StreamContext getInboundContext();
    /**
     * <p>
     * Retrieves the <tt>StreamContext</tt> for outbound operations. This
     * <tt>StreamContext</tt> should be referred to within {@link #send}.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The <tt>StreamContext</tt> for inbound operations.
     */
    public StreamContext getOutboundContext();
    
    /**
     * <p>
     * Retrieves the <tt>Status</tt> for this <tt>PacketRouter</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The current <tt>Status</tt>.
     * @since   JSO-0.10
     */
    public Stream.Status getCurrentStatus();
    
    /**
     * <p>
     * Retrieves the default namespace for application-level <tt>Packet</tt>s.
     * This namespace is what most <tt>Packet</tt> are expected to be qualified
     * by. This namespace is usually different from the inbound and oubound
     * <tt>StreamContext</tt>s.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The default namespace.
     * @since   JSO-0.12
     */
    public String getDefaultNamespace();
    
    /**
     * <p>
     * Send a <tt>Packet</tt> across this <tt>PacketRouter</tt>.</p>
     *
     * <p>
     * If the value of <tt>p</tt> is <tt>null</tt>, this method may trigger no
     * actions.</p>
     *
     * <p>
     * Any exception thrown (for reasons other than the above) is caught and
     * re-thrown as a <tt>StreamException</tt>. If this happens, this
     * <tt>PacketRouter</tt> may no longer be usable.</p>
     *
     * @param p The <tt>Packet</tt> to send.
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.10
     */
    public void send(Packet p) throws StreamException;

    /**
     * <p>
     * Processes <tt>Packet</tt> data coming into this
     * <tt>PacketRouter</tt>.</p>
     *
     * <p>
     * Any exception thrown is caught and re-thrown as a
     * <tt>StreamException</tt>. If this happens, this
     * <tt>PacketRouter</tt> may no longer be usable.</p>
     *
     * @throws StreamException If any errors are encountered.
     * @since   JSO-0.10
     */
    public void process() throws StreamException;
}