/*
 * StreamFeaturesDispatcher.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.event;

/**
 * <p>
 * "Registration" interface for objects that dispatch
 * <tt>StreamFeaturesEvent</tt>. This interface provides the methods to
 * register and unregister <tt>StreamFeaturesListener</tt> objects.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface StreamFeaturesDispatcher {
    //Methods
    /**
     * <p>
     * Registers the given <tt>StreamFeaturesListener</tt> for any features
     * trasfer events. The given listener is registered to events for both
     * received and sent <tt>StreamFeatureset</tt> objects.</p>
     *
     * <p>
     * The value of <tt>l</tt> should not be <tt>null</tt>.  No actions
     * are performed if this is true. If <tt>l</tt> is already registered,
     * it may again be registered.</p>
     *
     * @param l The <tt>StreamFeaturesListener</tt> to register.
     * @since   JSO-0.12
     */
    public void addStreamFeaturesListener(StreamFeaturesListener l);
    /**
     * <p>
     * Registers the given <tt>StreamFeaturesListener</tt> for packet transfer
     * events of the given type. The given listener is registered only to
     * events matching <tt>t</tt>.</p>
     *
     * <p>
     * The value of <tt>t</tt> can be <tt>null</tt>, in which case this
     * method performs identically to {@link
     * #addStreamFeaturesListener(StreamFeaturesListener)}.</p>
     *
     * <p>
     * The value of <tt>l</tt> should not be <tt>null</tt>.  No actions
     * are performed if this is true.  If <tt>l</tt> is already registered,
     * it is registered again.</p>
     *
     * @param type The <tt>PacketEvent.Type</tt> to register for.
     * @param l The <tt>StreamFeaturesListener</tt> to register.
     * @since   JSO-0.6
     */
    public void addStreamFeaturesListener(PacketEvent.Type type, StreamFeaturesListener l);
    
    /**
     * <p>
     * Unregisters the given <tt>StreamFeaturesListener</tt> for any packet
     * transfer events.  The given listener is unregistered from events for both
     * received and sent <tt>StreamFeatures</tt> objects.</p>
     *
     * <p>
     * The value of <tt>l</tt> should not be <tt>null</tt>.  No actions
     * are performed if this is true.  If <tt>l</tt> is registered more
     * than once, only one is unregistered.</p>
     *
     * @param l The <tt>StreamFeaturesListener</tt> to unregister.
     * @since   JSO-0.6
     */
    public void removeStreamFeaturesListener(StreamFeaturesListener l);
    /**
     * <p>
     * Unegisters the given <tt>StreamFeaturesListener</tt> for packet transfer
     * events of the given type.  The given listener is unregistered only from
     * events matching <tt>t</tt>.</p>
     *
     * <p>
     * The value of <tt>t</tt> can be <tt>null</tt>, in which case this
     * method performs identically to {@link
     * #removeStreamFeaturesListener(StreamFeaturesListener)}.</p>
     *
     * <p>
     * The value of <tt>l</tt> should not be <tt>null</tt>.  No actions
     * are performed if this is true.  If <tt>l</tt> is registered more
     * than once, only one is unregistered.</p>
     *
     * @param type The <tt>PacketEvent.Type</tt> to register for.
     * @param l The <tt>StreamFeaturesListener</tt> to register.
     * @since   JSO-0.6
     */
    public void removeStreamFeaturesListener(PacketEvent.Type type, StreamFeaturesListener l);
}