/*
 * LocaleFormat.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.format;


import java.text.AttributedCharacterIterator;
import java.text.AttributedString;
import java.text.FieldPosition;
import java.text.Format;
import java.text.ParseException;
import java.text.ParsePosition;
import java.util.Locale;
import java.util.Map;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Formats and parses <tt>Locale</tt> objects to and from the xml:lang
 * attribute format.</p>
 *
 * <p>
 * The expected pattern is "&lt;language code&gt;" or
 * "&lt;language-code&gt;-&lt;country-code&gt;", where &lt;language-code&gt;
 * is the two-letter ISO language code (e.g. English is 'en') and
 * &lt;country-code&gt; is the two-letter ISO country code (e.g. United
 * States of America is 'US').</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class LocaleFormat extends Format {
    //Inner-classes
    private static class Field extends Format.Field {
        //Constructors
        public Field(String name) {
            super(name);
        }
    }
    
    //"Constants"
    /** Constant for "language-code" field alignment. Used in
     * <tt>FieldPosition</tt> in formatting.</p>
     *
     * @since   JSO-0.12
     */
    public static final Field       LANGUAGE_FIELD  = new Field("language");
    /** Constant for "country-code" field alignment. Used in
     * <tt>FieldPosition</tt> in formatting.</p>
     *
     * @since   JSO-0.12
     */
    public static final Field       COUNTRY_FIELD   = new Field("country");
    
    private static final Pattern    RE_XMLLANG  = Pattern.compile("(\\w{1,8})(-(\\w{1,8}))?");

    //Fields
    private static final LocaleFormat       _Instance = new LocaleFormat();
    
    //Constructors
    private LocaleFormat() {
    }

    //Methods
    /**
     * <p>
     * Formats the given <tt>Locale</tt> into an xml:lang string.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt> or "".</p>
     *
     * <p>
     * The value of <tt>obj</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param obj The locale to format.
     * @return The xml:lang string.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public String format(Locale l) throws IllegalArgumentException {
        return format((Object)l);
    }
    /**
     * <p>
     * Formats the object as an xml:lang string.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>obj</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown. Also, if <tt>obj</tt> is
     * not an instance of <tt>Locale</tt>, an <tt>IllegalArgumentException</tt>
     * is thrown.</p>
     *
     * <p>
     * The value of <tt>toAppendTo</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>pos</tt> may be <tt>null</tt>, in which case it is
     * ignored.</p>
     *
     * @param obj The object to format.
     * @param toAppendTo The string buffer to append to.
     * @param pos A <tt>FieldPosition</tt> identifying a field in the formatted
     * text.
     * @return The string buffer passed in as <tt>toAppendTo</tt>, with the
     * formatted text appended.
     * @throws NullPointerException If <tt>toAppendTo</tt> is <tt>null</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public StringBuffer format(Object obj, StringBuffer toAppendTo, FieldPosition pos) throws IllegalArgumentException {
        AttributedCharacterIterator itr = formatToCharacterIterator(obj, toAppendTo);
        
        if (pos != null) {
            if (pos.getFieldAttribute() == LANGUAGE_FIELD) {
                pos.setBeginIndex(itr.getRunStart(LANGUAGE_FIELD));
                pos.setEndIndex(itr.getRunLimit(LANGUAGE_FIELD));
            }
            itr.setIndex(itr.getRunStart(LANGUAGE_FIELD));
            
            if (pos.getFieldAttribute() == COUNTRY_FIELD) {
                pos.setBeginIndex(itr.getRunStart(COUNTRY_FIELD));
                pos.setEndIndex(itr.getRunLimit(COUNTRY_FIELD));
            }
            itr.setIndex(itr.getRunStart(COUNTRY_FIELD));
        }

        return toAppendTo;
    }
    /**
     * <p>
     * Formats a Locale into an <tt>AttributedStringIterator</tt> as an
     * xml:lang string. The returned iterator can be used to build the
     * resulting string, as well as to determine information about the
     * resulting string.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each attribute
     * in the returned iterator is an instance of type
     * <tt>Format.Field</tt>. The order to examine the attributes is:</p>
     * <ol>
     * <li>{@link #LANGUAGE_FIELD}</li>
     * <li>{@link #COUNTRY_FIELD}</li>
     * </ol>
     *
     * <p>
     * The value of <tt>obj</tt> must be a non-<tt>null</tt> instance of
     * <tt>Locale</tt>, or an <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param obj The locale to format.
     * @return The <tt>AttributedCharacterIterator</tt> describing the resulting
     * string.
     * @throws IllegalArgumentException If the paramter is invalid.
     * @since   JSO-0.12
     */
    public AttributedCharacterIterator formatToCharacterIterator(Object obj) throws IllegalArgumentException {
        return formatToCharacterIterator(obj, new StringBuffer());
    }
    private AttributedCharacterIterator formatToCharacterIterator(Object obj, StringBuffer toAppendTo) throws IllegalArgumentException {
        Map                 attrMap;
        AttributedString    attrStr;
        StringBuffer        buffer = new StringBuffer();
        Locale              l;
        String              lang, cn;
        int                 start = toAppendTo.length(), finish;
        int[]               begin = new int[2], end = new int[2];
        
        if (obj == null)
            throw new IllegalArgumentException("object cannot be null");
        if (!(obj instanceof Locale))
            throw new IllegalArgumentException("object must be an instance of Locale");
        l = (Locale)obj;
        attrMap = new java.util.HashMap();

        lang = l.getLanguage();
        cn = l.getCountry();

        begin[0] = start + buffer.length();
        buffer.append(lang);
        end[0] = start + buffer.length();
        attrMap.put(LANGUAGE_FIELD, lang);
        
        if (Utilities.isValidString(cn)) {
            begin[1] = start + buffer.length();
            buffer.append('-').append(cn);
            end[1] = start + buffer.length();
            attrMap.put(COUNTRY_FIELD, cn);
        }
        finish = start + buffer.length();
        toAppendTo.append(buffer);
        
        attrStr = new AttributedString(buffer.toString(), attrMap);
        if (Utilities.isValidString(lang))
            attrStr.addAttribute(LANGUAGE_FIELD, lang, begin[0], end[0]);
        if (Utilities.isValidString(cn))
            attrStr.addAttribute(COUNTRY_FIELD, cn, begin[1], end[1]);

        return attrStr.getIterator();
    }
    
    /**
     * <p>
     * Parses text from a xml:lang string into a <tt>Locale</tt>.</p>
     *
     * <p>
     * If parsing fails, then a <tt>ParseException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>source</tt> may be <tt>null</tt>, in which case parsing
     * always fails.</p>
     *
     * @param source The string to convert.
     * @return The Locale parsed from <tt>source</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public Locale parse(String source) throws ParseException {
        return (Locale)parseObject(source);
    }
    /**
     * <p>
     * Parses text from a xml:lang string into a <tt>Locale</tt>.</p>
     *
     * <p>
     * If parsing fails, then the error index of <tt>pos</tt> is set to the
     * initial parsing index, and <tt>null</tt> is returned.</p>
     *
     * <p>
     * The value of <tt>source</tt> may be <tt>null</tt>, in which case parsing
     * always fails.</p>
     *
     * <p>
     * The value of <tt>pos</tt> cannot be <tt>null</tt>, or a
     * <tt>NullPointerException</tt> is thrown.</p>
     *
     * @param source The string to convert.
     * @param pos A <tt>ParsePosition</tt> object with the index and error
     * index information.
     * @return The Locale parsed from <tt>source</tt>.
     * @throws NullPointerException If <tt>pos</tt> is <tt>null</tt>.
     * @since   JSO-0.12
     */
    public Object parseObject(String source, ParsePosition pos) throws NullPointerException {
        Locale  lang = null;
        Matcher match;
        
        //Validate input
        if (source == null)
            source = "";

        try {
            match = RE_XMLLANG.matcher(source.substring(pos.getIndex()));
            if (match.find() && (match.start() == 0)) {
                String  pri = match.group(1), sec = match.group(3);

                lang = (Utilities.isValidString(sec) ? new Locale(pri, sec) : new Locale(pri));
                pos.setIndex(match.end() + pos.getIndex());
            } else {
                pos.setErrorIndex(pos.getIndex());
            }
        } catch (NullPointerException npe) {
            throw npe;
        } catch (Exception e) {
            pos.setErrorIndex(pos.getIndex());
        }

        return lang;
    }
    
    /**
     * <p>
     * Retrieves the default <tt>LocaleFormat</tt> instance.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The default <tt>LocaleFormat</tt> instance.
     * @since   JSO-0.12
     */
    public static LocaleFormat getInstance() {
        return _Instance;
    }
}