/*
 * HttpPollingStreamSource.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.io.src;


import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.ByteBuffer;
import java.util.Random;

import org.apache.log4j.Logger;

import org.jabberstudio.jso.Stream;
import org.jabberstudio.jso.io.StreamSource;
import org.jabberstudio.jso.util.ByteCodec;
import org.jabberstudio.jso.util.DigestHash;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Implementation of <tt>StreamSource</tt> to support HTTP-Polling, as defined
 * in JEP-0025.</p>
 *
 * <p><b>WARNING:</b> This <tt>StreamSource</tt> should only be used for
 * initiating connections (e.g. clients). Using this <tt>StreamSource</tt> for
 * any other purpose is not supported.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 * @see <a href='http://www.jabber.org/jeps/jep-0025.html'>JEP-0025: HTTP Polling</a>
 */
public class HttpPollingStreamSource implements StreamSource {
    //Constants
    private final static int        DEFAULT_ITERATIONS = 200;
    private final static String     UNKNOWN_ERROR = "0:0";
    private final static String     SERVER_ERROR = "-1:0";
    private final static String     BAD_REQUEST = "-2:0";

    //Fields
    private URL         _Loc;
    private String      _ID;
    private String[]    _Keys;
    private int         _KeyIdx;
    
    private ByteBuffer  _Buffer;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>HttpPollingStreamSource</tt> with the given URL.</p>
     *
     * <p>
     * The value of <tt>loc</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param loc The location to poll.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public HttpPollingStreamSource(URL loc) throws IllegalArgumentException {
        setLocation(loc);
    }

    //Methods
    /**
     * <p>
     * Retrieves the location this <tt>HttpPollingStreamSource</tt> is
     * polling.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The location to poll.
     * @since   JSO-0.12
     */
    public URL getLocation() {
        return _Loc;
    }
    private void setLocation(URL loc) throws IllegalArgumentException {
        if (loc == null)
            throw new IllegalArgumentException("URL location cannot be null");
        _Loc = loc;
    }
    
    public String getHostname() {
        return getLocation().getHost();
    }
    
    public void connect(Stream s) throws IOException, Exception {
    }
    public void disconnect(Stream s) throws IOException, Exception {
    }
    
    public boolean ready() throws IOException {
        return true;
    }
    public int read(byte[] data, int offset, int length) throws IllegalArgumentException, IOException {
        ByteBuffer      buffer = obtainBuffer();
        int             amt = buffer.remaining();
        
        if (length < buffer.remaining())
            amt = length;
        buffer.get(data, offset, amt);
        
        return length;
    }
    public int write(byte[] data, int offset, int length) throws IllegalArgumentException, IOException {
        byte[]      in = poll(data, offset, length);
        
        if ((in != null) && (in.length > 0))
            fillBuffer(in);
        
        return length;
    }

    private final synchronized ByteBuffer getBuffer() {
        if (_Buffer == null) {
            _Buffer = ByteBuffer.allocate(Short.MAX_VALUE);
            _Buffer.position(0).limit(0);
        }
        
        return _Buffer;
    }
    private final synchronized ByteBuffer obtainBuffer() throws IOException {
        ByteBuffer      buffer = getBuffer();
        byte[]          data;
        
        if (!buffer.hasRemaining()) {
            data = poll();
            buffer = fillBuffer(data);
        }
        
        return buffer;
    }
    private final synchronized ByteBuffer fillBuffer(byte[] data) throws IOException {
        ByteBuffer  buffer;
        int         size = data.length + (_Buffer.remaining());
        
        buffer = ByteBuffer.allocate(size);
        buffer.put(_Buffer);
        buffer.put(data);
        _Buffer = buffer;
        
        return _Buffer;
    }

    private byte[] poll() throws IOException {
        return poll(null, 0, 0);
    }
    private byte[] poll(byte[] data, int offset, int length) throws IOException {
        HttpURLConnection       conn;
        ByteArrayOutputStream   bout;
        StringBuffer            buffer;
        String                  cookie, type;
        OutputStream            out;
        InputStream             in;
        
        conn = obtainConnection();
        bout = new ByteArrayOutputStream(Short.MAX_VALUE);

        //Output request
        out = conn.getOutputStream();
        buffer = createHeader();
        bout.write(buffer.toString().getBytes("UTF8"));
        if ((data != null) && (data.length > 0) && (offset < data.length) && (length <= data.length)) {
            bout.write(data, offset, length);
            bout.write("\r\n".getBytes("UTF8"));
        }
        out.write(bout.toByteArray());
        out.flush();
        out.close();
        bout.reset();
        
        //Input response
        in = conn.getInputStream();
        cookie = conn.getHeaderField("Set-Cookie");
        if (Utilities.isValidString(cookie)) {
            cookie = cookie.substring(cookie.indexOf('=') + 1, cookie.indexOf(';'));
            
            if      (Utilities.equateStrings(cookie, BAD_REQUEST))
                throw new IOException("server could not understand request");
            else if (Utilities.equateStrings(cookie, SERVER_ERROR))
                throw new IOException("server error");
            else if (Utilities.equateStrings(cookie, UNKNOWN_ERROR))
                throw new IOException("unknown error");
            else
                setConnectionID(cookie);
        }
        if (Utilities.equateStrings(conn.getContentType(), "text/xml")) {
            byte[]      temp = new byte[Short.MAX_VALUE];
            int         amt;
            
            while ((amt = in.read(temp, 0, temp.length)) != -1)
                bout.write(temp, 0, amt);
        } else {
            throw new IOException("content-type not acceptable");
        }
        
        return bout.toByteArray();
    }
    private HttpURLConnection obtainConnection() throws IOException {
        URL                 url = getLocation();
        HttpURLConnection   conn = (HttpURLConnection)url.openConnection();
        
        conn.setRequestMethod("POST");
        conn.setUseCaches(false);
        conn.setDoOutput(true);
        conn.setDoInput(true);
        conn.setAllowUserInteraction(false);
        conn.setRequestProperty("content-type", "application/x-www-form-urlencoded");
        
        return conn;
    }
    
    private final String getConnectionID() {
        if (_ID == null)
            _ID = "0";
        return _ID;
    }
    private final void setConnectionID(String id) {
        _ID = id;
    }

    private final synchronized StringBuffer createHeader() {
        StringBuffer    header = new StringBuffer(getConnectionID());
        String[]        keys = getKeylist();

        header.append(';').append(_Keys[_KeyIdx--]);
        if (_KeyIdx == 0) {
            keys = resetKeys();
            header.append(';');
            header.append(keys[_KeyIdx]);
        }
        header.append(',');
        
        return header;
    }
    private final String[] getKeylist() {
        if (_Keys == null)
            resetKeys();
        
        return _Keys;
    }
    private final synchronized String[] resetKeys() {
        DigestHash      sha1 = DigestHash.SHA1;
        ByteCodec       b64 = ByteCodec.BASE64;
        Random          random = new java.security.SecureRandom();
        byte            working[] = new byte[20];

        _Keys = new String[DEFAULT_ITERATIONS];

        random.nextBytes(working);
        for (int i = 0; i < _Keys.length; i++) {
            _Keys[i] = b64.encode(working);

            try {
                working = sha1.hashToBytes(_Keys[i], "UTF8");
            } catch (UnsupportedEncodingException e) {
                obtainLogger().warn("UTF8 encoding not supported", e);
            }
        }
        _KeyIdx = _Keys.length - 1;
        
        return _Keys;
    }

    private final Logger obtainLogger() {
        return Logger.getLogger(getClass());
    }
}