/*
 * ConfirmAuthorizationCallback.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.sasl.callback;


import org.jabberstudio.jso.JID;

/**
 * <p>
 * Underlying security services instantiate and pass a
 * <tt>ConfirmAuthorizationCallback</tt> to the <tt>handle</tt> method of a
 * <tt>CallbackHandler</tt> to confirm one entity (identified by the
 * authentication ID) is allowed to act on behalf of another (identified by the
 * authorization ID).</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public class ConfirmAuthorizationCallback implements javax.security.auth.callback.Callback {
    //Fields
    private JID     _Authnid;
    private JID     _Authzid;
    
    private boolean _Confirm = false;
    
    //Constructors
    /**
     * <p>
     * Creates a new <tt>AuthorizationIDCallback</tt> with the given
     * authentication ID and authorization ID.</p>
     *
     * <p>
     * The value of <tt>authn</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>authz</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param authn The authentication ID (as a JID).
     * @param authz The authorization ID (as a JID).
     * @throws IlegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public ConfirmAuthorizationCallback(JID authn, JID authz) throws IllegalArgumentException {
        if (authn == null)
            throw new IllegalArgumentException("Authentication ID cannot be null");
        if (authz == null)
            throw new IllegalArgumentException("Authorization ID cannot be null");
        
        _Authnid = authn;
        _Authzid = authz;
    }
    
    //Methods
    /**
     * <p>
     * Retrieves the authentication ID.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The authentication ID.
     * @since   JSO-0.12
     */
    public JID getAuthenticationID() {
        return _Authnid;
    }
    /**
     * <p>
     * Retrieves the authorization ID.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The authorization ID.
     * @since   JSO-0.12
     */
    public JID getAuthorizationID() {
        return _Authnid;
    }

    /**
     * <p>
     * Determines if the authentication ID is allowed to act on behalf of the
     * authorization ID.</p>
     *
     * @return <tt>true</tt> if {@link #getAuthenticationID} can act on behalf
     * of {@link #getAuthorizationID}.
     * @since   JSO-0.12
     */
    public boolean isAuthorized() {
        return _Confirm;
    }
    /**
     * <p>
     * Sets the flag determing if the authentication ID is allowed to act on
     * behalf of the authorization ID.</p>
     *
     * @param conf <tt>true</tt> if authorization is allowed.
     * @since   JSO-0.12
     */
    public void setAuthorized(boolean conf) {
        _Confirm = conf;
    }
}