/*
 * ByteCodec.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.util;


import java.io.ByteArrayOutputStream;

/**
 * <p>
 * Interface for converting bytes to encoding strings, and vice versa. The
 * interface includes default references to codecs for {@link #BASE64 Base-64}
 * and {@link #HEX hexidecimal} formats.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface ByteCodec {
    //Inner-classes
    /**
     * <p>
     * Implementation of <tt>ByteCode</tt> to encode and decode using the
     * hexidecimal format.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     */
    public static class HexCodec implements ByteCodec {
        //Fields
        public static final String  DICTIONARY = "0123456789abcdef";
        
        /**
         * <p>
         * Encodes the given byte array into a string.</p>
         *
         * <p>
         * The value of <tt>input</tt> may be <tt>null</tt>, in which case it
         * is considered to be a zero-length array.</p>
         *
         * @param input The bytes to convert.
	 * @param offset index of first array element to convert.
	 * @param len number of bytes to convert
         * @return The encoded string.
         * @since   JSO-0.12
         */
        public String encode(byte[] input, int offset, int len) {
            StringBuffer    output = new StringBuffer();
            int             idx;
            
            if (input == null)
                input = new byte[0];

	    len += offset;
            if (len > input.length)
                len = input.length;
            
            for (idx = offset; idx < len; idx++) {
                output.append(DICTIONARY.charAt((input[idx] >> 4) & 0x0f));
                output.append(DICTIONARY.charAt(input[idx] & 0x0f));
            }
            
            return output.toString();
        }
        /**
         * <p>
         * Encodes the given byte array into a string.</p>
         *
         * <p>
         * The value of <tt>input</tt> may be <tt>null</tt>, in which case it
         * is considered to be a zero-length array.</p>
         *
         * @param input The bytes to convert.
         * @return The encoded string.
         * @since   JSO-0.12
         */
        public String encode(byte[] input) {
            return encode(input, 0, input.length);
        }
        /**
         * <p>
         * Decodes the given string into a byte array.</p>
         *
         * <p>
         * The value of <tt>input</tt> may be <tt>null</tt> or "", in which case
         * it is considered to be "".</p>
         *
         * @param input The string to convert.
         * @return The decoded bytes.
         * @throws IllegalArgumentException If <tt>input</tt> cannot be converted
         * to bytes.
         * @since   JSO-0.12
         */
        public byte[] decode(String input) throws IllegalArgumentException {
            ByteArrayOutputStream   output = new ByteArrayOutputStream();
            byte                    data;
            int                     idx, outPos;
            char                    ch = 0;
            
            if (input == null)
                input = "";
            
            data = 0;
            outPos = 0;
            try {
                for (idx = 0; idx < input.length(); idx++) {
                    ch = input.charAt(idx);

                    if (Character.isWhitespace(ch))
                        continue;

                    outPos++;
                    data <<= 4;
                    data += (byte)DICTIONARY.indexOf(Character.toLowerCase(ch));
                    if ((outPos % 2) == 0) {
                        output.write(data);

                        data = 0;
                    }
                }
            } catch (IndexOutOfBoundsException iobe) {
                throw new IllegalArgumentException("Illegal character in encoded string: \'" + ch + "\'");
            }
            
            //Verify encoded length
            if ((outPos % 2) != 0)
                throw new IllegalArgumentException("Invalid length for encoded string");
            
            return output.toByteArray();
        }
    }
    
    /**
     * <p>
     * Implementation of <tt>ByteCode</tt> to encode and decode using the
     * base-64 format.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     */
    public static class Base64Codec implements ByteCodec {
        //"Constants"
        private static final String DICTIONARY = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

        //Methods
        /**
         * <p>
         * Encodes the given byte array into a string.</p>
         *
         * <p>
         * The value of <tt>input</tt> may be <tt>null</tt>, in which case it
         * is considered to be a zero-length array.</p>
         *
         * @param input The bytes to convert.
         * @return The encoded string.
         * @since   JSO-0.12
         */
        public String encode(byte[] input) {
	    return encode(input, 0, input.length);
	}
        /**
         * <p>
         * Encodes the given byte array into a string.</p>
         *
         * <p>
         * The value of <tt>input</tt> may be <tt>null</tt>, in which case it
         * is considered to be a zero-length array.</p>
         *
         * @param input The bytes to convert.
	 * @param offset index of first array element to convert.
	 * @param len number of bytes to convert
         * @return The encoded string.
         * @since   JSO-0.12
         */
        public String encode(byte[] input, int offset, int len) {
            StringBuffer    output = new StringBuffer();
            int             temp, packed;
            int		    rem, l;
            int             idx, pos1, pos2, pos3;

           //Validate input
            if (input == null)
                input = new byte[0];

            //Prep fields
            rem = len % 3;

	    len += offset;
            if (len > input.length)
                len = input.length;

            l = len - rem;

            //Process complete triples
            for (idx = offset; idx < l; idx += 3) {
                packed = ((input[idx] & 0xff) << 16) | ((input[idx + 1] & 0xff) << 8) | (input[idx + 2] & 0xff);
                
                output.append(DICTIONARY.charAt((packed >>> 18) & 0x3f));
                output.append(DICTIONARY.charAt((packed >>> 12) & 0x3f));
                output.append(DICTIONARY.charAt((packed >>> 6) & 0x3f));
                output.append(DICTIONARY.charAt(packed & 0x3f));
            }

            //Process partial triple
            switch (rem) {
                case 1:
                    packed = ((input[idx] & 0xff) << 16);

                    output.append(DICTIONARY.charAt((packed >>> 18) & 0x3f));
                    output.append(DICTIONARY.charAt((packed >>> 12) & 0x3f));
                    output.append('=');
                    output.append('=');
                    break;
                case 2:
                    packed = ((input[idx] & 0xff) << 16) | ((input[idx + 1] & 0xff) << 8);

                    output.append(DICTIONARY.charAt((packed >>> 18) & 0x3f));
                    output.append(DICTIONARY.charAt((packed >>> 12) & 0x3f));
                    output.append(DICTIONARY.charAt((packed >>> 6) & 0x3f));
                    output.append('=');
                    break;
            }

            return output.toString();
        }
        /**
         * <p>
         * Decodes the given string into a byte array.</p>
         *
         * <p>
         * The value of <tt>input</tt> may be <tt>null</tt> or "", in which case
         * it is considered to be "".</p>
         *
         * @param input The string to convert.
         * @return The decoded bytes.
         * @throws IllegalArgumentException If <tt>input</tt> cannot be converted
         * to bytes.
         * @since   JSO-0.12
         */
        public byte[] decode(String input) throws IllegalArgumentException {
            ByteArrayOutputStream   output = new ByteArrayOutputStream();
            char                    ch = 0;
            int                     idx, outPos;
            int                     rem, pad;
            int                     quad;

            //Validate input
            if (input == null)
                input = "";

            //Prep fields
            quad = 0;
            outPos = 0;
            pad = 0;

            try {
                //Process complete quads
                for (idx = 0; idx < input.length(); idx++) {
                    ch = input.charAt(idx);

                    if (Character.isWhitespace(ch))
                        continue;

                    quad <<= 6;
                    if (ch == '=') {
                        pad++;
                    } else {
                        outPos++;
                        quad += (DICTIONARY.indexOf(ch) & 0x3f);
                        if ((outPos % 4) == 0) {
                            output.write((quad >> 16) & 0xff);
                            output.write((quad >> 8)  & 0xff);
                            output.write(quad & 0xff);

                            quad = 0;
                        }
                    }
                }
            } catch (IndexOutOfBoundsException iobe) {
                throw new IllegalArgumentException("Illegal character in encoded string: \'" + ch + "\'");
            }
            
            //Verify string is proper length
            if (((outPos + pad) % 4) != 0)
                throw new IllegalArgumentException("Invalid length for encoded string");

            //Process partial quad
            if (pad > 0) {
                output.write((quad >> 16) & 0xff);
                if (pad < 2) {
                    output.write((quad >> 8) & 0xff);
                }
            }

            return output.toByteArray();
        }
    }
    
    //"Constants"
    /**
     * Codec for converting to and from hexidecimal format.
     *
     * @since   JSO-0.12
     */
    public static final ByteCodec   HEX = new ByteCodec.HexCodec();
    /**
     * Codec for converting to and from base-64 format.
     *
     * @since   JSO-0.12
     */
    public static final ByteCodec   BASE64 = new Base64Codec();
    
    //Methods
    /**
     * <p>
     * Encodes the given byte array into a string.</p>
     *
     * <p>
     * The value of <tt>input</tt> may be <tt>null</tt>, in which case it
     * is considered to be a zero-length array.</p>
     *
     * @param input The bytes to convert.
     * @return The encoded string.
     * @since   JSO-0.12
     */
    public String encode(byte[] input);
    /**
     * <p>
     * Encodes the given byte array into a string.</p>
     *
     * <p>
     * The value of <tt>input</tt> may be <tt>null</tt>, in which case it
     * is considered to be a zero-length array.</p>
     *
     * @param input Array of bytes containg the bytes to convert.
     * @param offset index of first array element to convert.
     * @param len number of bytes to convert
     * @return The encoded string.
     * @since   JSO-0.12
     */
    public String encode(byte[] input, int offset, int len);
    /**
     * <p>
     * Decodes the given string into a byte array.</p>
     *
     * <p>
     * The value of <tt>input</tt> may be <tt>null</tt> or "", in which case
     * it is considered to be "".</p>
     *
     * @param input The string to convert.
     * @return The decoded bytes.
     * @throws IllegalArgumentException If <tt>input</tt> cannot be converted
     * to bytes.
     * @since   JSO-0.12
     */
    public byte[] decode(String input) throws IllegalArgumentException;
}
