/*
 * DigestHash.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.util;


import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;

import org.jabberstudio.jso.util.ByteCodec;

/**
 * <p>
 * Class to generate "secure hash" strings.  This class makes use of
 * <code>java.security.MessageDigest</code> to generate the hashes.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.2
 */
public class DigestHash {
    //"Constants"
    /** <code>DigestHash</code> for the SHA1 algorithm.
     *
     * @since   JSO-0.2
     */
    public static final DigestHash  SHA1;
    /** <code>DigestHash</code> for the MD5 algorithm.
     *
     * @since   JSO-0.2
     */
    public static final DigestHash  MD5;
    
    static {
        DigestHash  digest;

        try {
            digest = new DigestHash(MessageDigest.getInstance("sha1"));
        } catch (Exception e) {
            digest = null;
        }
        SHA1 = digest;

        try {
            digest = new DigestHash(MessageDigest.getInstance("md5"));
        } catch (Exception e) {
            digest = null;
        }
        MD5 = digest;
    }
    
    //Fields
    private MessageDigest   _Digest;
    
    //Constructors
    /**
     * <p>
     * Creates a new <code>DigestHash</code> using the given
     * <code>MessageDigest</code>.</p>
     *
     * <p>
     * The value of <code>digest</code> cannot be <code>null</code>, or an
     * <code>IllegalArgumentException</code> is thrown.</p>
     *
     * @param digest The <code>MessageDigest</code> to use.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.2
     */
    public DigestHash(MessageDigest digest) throws IllegalArgumentException {
        setDigest(digest);
    }
    
    //Methods
    protected MessageDigest getDigest() {
        return _Digest;
    }
    private void setDigest(MessageDigest digest) throws IllegalArgumentException {
        if (digest == null)
            throw new IllegalArgumentException("MessageDigest cannot be null");
        _Digest = digest;
    }
    
    /**
     * <p>
     * Generates a "secure hash" string for the given input string.  This
     * method resets the internal <tt>MessageDigest</tt> before each usage, and
     * returns a string converted to the hexidecimal-equivalent of the
     * generated hash. The input string is converted to bytes using the
     * character set "UTF-8".</p>
     *
     * <p>
     * The value of <tt>input</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param input The input string to hash.
     * @return The hash of the input string.
     * @since   JSO-0.2
     */
    public synchronized String hash(String input) {
        String  val;
        
        try {
            val = hash(input, "UTF-8", null);
        } catch (UnsupportedEncodingException uee) {
            val = "";
        }
        
        return val;
    }
    /**
     * <p>
     * Generates a "secure hash" string for the given input string. This method
     * resets the internal <tt>MessageDigest</tt> before each usage, and
     * returns a string converted to the hexidecimal-equivalent of the
     * generated hash. The input string is converted to bytes using the
     * named character set.</p>
     *
     * <p>
     * The value of <tt>input</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * <p>
     * The value of <tt>charset</tt> must be the name of a supported character
     * set for decoding, or an <tt>UnsupportedEncodingException</tt> is
     * thrown.</p>
     *
     * @param input The input string to hash.
     * @return The hash of the input string.
     * @throws UnsupportedEncodingException If the named character is not
     * supported.
     * @since   JSO-0.12
     */
    public synchronized String hash(String input, String charset) throws UnsupportedEncodingException {
        return hash(input, charset, null);
    }
    /**
     * <p>
     * Generates a "secure hash" string for the given input string. This method
     * resets the internal <tt>MessageDigest</tt> before each usage, and
     * returns a string converted to a string using the given codec. The input
     * string is converted to bytes using the named character set.</p>
     *
     * <p>
     * The value of <tt>input</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * <p>
     * The value of <tt>charset</tt> must be the name of a supported character
     * set for decoding, or an <tt>UnsupportedEncodingException</tt> is
     * thrown.</p>
     *
     * <p>
     * The value of <tt>codec</tt> may be <tt>null</tt>, in which case
     * {@link ByteCodec#HEX} is used.</p>
     *
     * @param input The input string to hash.
     * @param charset The character set name to convert the input string to
     * bytes.
     * @param codec The <tt>ByteCodec</tt> to convert the output bytes to a
     * string.
     * @return The hash of the input string.
     * @throws UnsupportedEncodingException If the named character is not
     * supported.
     * @since   JSO-0.12
     */
    public synchronized String hash(String input, String charset, ByteCodec codec) throws UnsupportedEncodingException {
        byte[]          inBytes, outBytes;
        StringBuffer    output = new StringBuffer();
        int             idx;
        
        //Validate input
        if (codec == null)
            codec = ByteCodec.HEX;

        //Prepare and make digest (bytes)
        outBytes = hashToBytes(input, charset);

        //Convert digest bytes to <codec>-string
        return codec.encode(outBytes);
    }
    
    /**
     * <p>
     * Generates a "secure hash" byte array for the given input string. This
     * method resets the internal <tt>MessageDigest</tt> before each usage. The
     * input string is converted to bytes using the character set "UTF-8".</p>
     *
     * <p>
     * The value of <tt>input</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param input The input string to hash.
     * @return The hash bytes of the input string.
     * @throws UnsupportedEncodingException If the named character is not
     * supported.
     * @since   JSO-0.12
     */
    public synchronized byte[] hashToBytes(String input) {
        byte[]      data;
        
        try {
            data = hashToBytes(input, "UTF-8");
        } catch (UnsupportedEncodingException uee) {
            data = new byte[0];
        }
        
        return data;
    }
    /**
     * <p>
     * Generates a "secure hash" byte array for the given input string. This
     * method resets the internal <tt>MessageDigest</tt> before each usage. The
     * input string is converted to bytes using the named character set.</p>
     *
     * <p>
     * The value of <tt>input</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * <p>
     * The value of <tt>charset</tt> must be the name of a supported character
     * set for decoding, or an <tt>UnsupportedEncodingException</tt> is
     * thrown.</p>
     *
     * @param input The input string to hash.
     * @return The hash bytes of the input string.
     * @throws UnsupportedEncodingException If the named character is not
     * supported.
     * @since   JSO-0.12
     */
    public synchronized byte[] hashToBytes(String input, String charset) throws UnsupportedEncodingException {
        byte[]          inBytes, outBytes;
        StringBuffer    output = new StringBuffer();
        int             idx;
        
        //Validate input
        if (input == null)
            input = "";

        //Prepare and make digest (bytes)
        getDigest().reset();
        inBytes = input.getBytes(charset);
        outBytes = getDigest().digest(inBytes);

        return outBytes;
    }
}