/*
 * DomainprepProfile.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */


package org.jabberstudio.jso.util.stringprep;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.jabberstudio.jso.util.Utilities;

/**
 * <p>
 * Implementation of <tt>StringprepProfile</tt> to prepare and verify domain
 * names. This implementation splits its input string (on dot boundaries) into
 * domain name labels, uses <tt>NameprepProfile</tt> to prepare each label, and
 * verifies each label does not contain unallowed characters (as defined by
 * STD 3).</p>
 *
 * @see     <a href='http://www.ietf.org/rfc/rfc1122.txt'>STD 3, RFC 1122: Requirements for Internet Hosts -- Communication Layers</a>
 * @see     <a href='http://www.ietf.org/rfc/rfc1123.txt'>STD 3, RFC 1122: Requirements for Internet Hosts -- Application and Support</a>
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12.3
 */
public class DomainprepProfile extends StringprepProfile {
    //"Constants"
    private static final Pattern            RE_LABEL_SEPERATOR  = Pattern.compile("[\\u002e\\u3002\\uff0e\\uff61]");
    private static final ProhibitedTable    STD3ASCII = ProhibitedTable.create(new char[][] {
        {0, 44},
        {46, 47},
        {58, 64},
        {91, 96},
        {123, 127}
    });

    //Fields
    private static DomainprepProfile  _Inst = new DomainprepProfile();

    //Constructors
    /**
     * <p>
     * Creates a new <tt>DomainprepProfile</tt>.</p>
     *
     * @since   JSO-0.12.3
     */
    protected DomainprepProfile() {
        super(  true,
                true,
                new MappingTable[] {
                    MAPPING_B_1,
                    MAPPING_B_2
                },
                new ProhibitedTable[] {
                    PROHIBITED_C_1_2,
                    PROHIBITED_C_2_2,
                    PROHIBITED_C_3,
                    PROHIBITED_C_4,
                    PROHIBITED_C_5,
                    PROHIBITED_C_6,
                    PROHIBITED_C_7,
                    PROHIBITED_C_8,
                    STD3ASCII
                });
    }

    //Methods
    /**
     * <p>
     * Prepares a string using this profile. This implementation uses the
     * following algorithm to prepare the input string:</p>
     *
     * <ol>
     * <li>Splits the input string to domain labels.</li>
     * <li>Prepares each label using the <tt>NameprepProfile</tt>.</li>
     * <li>Verifies each label does not contain ASCII characters not allowed by
     * STD 3.</li>
     * <li>Reassembles the string, preserving the original label
     * separators.</li>
     * </ol>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * If <tt>in</tt> is <tt>null</tt>, a <tt>StringprepException</tt> is
     * thrown.</p>
     *
     * @param   in The string to prepare.
     * @return  The prepared string.
     * @throws  StringprepException If there are any errors while preparing
     * <tt>in</tt>
     * @since   JSO-0.12.3
     */
    public String prepare(String in) throws StringprepException {
        Matcher         matcher;
        String          label;
        StringBuffer   out = new StringBuffer();
        int             start = 0, end;

        if (in == null)
            throw new StringprepException("input string cannot be null");

        matcher = RE_LABEL_SEPERATOR.matcher(in);
        while (matcher.find()) {
            end = matcher.start();
            label = in.substring(start, end);
            label = prepLabel(label);
            start = end + 1;

            out.append(label).append(matcher.group());
        }
        label = in.substring(start);
        label = prepLabel(label);
        out.append(label);

        return out.toString();
    }
    protected StringBuffer prohibited(StringBuffer buffer) throws ProhibitedException {
        char    first, last;

        buffer = super.prohibited(buffer);
        last = buffer.charAt(buffer.length() - 1);
        if ((buffer.charAt(0) == '-') || (buffer.charAt(buffer.length() - 1) == '-'))
            throw new ProhibitedException("cannot start or end with '-'");

        return buffer;
    }

    private final String prepLabel(String label) throws StringprepException {
        if (!Utilities.isValidString(label))
            throw new StringprepException("domain label cannot be null or \"\"");
        return super.prepare(label);
    }

    /**
     * <p>
     * Retrieves the global instance of <tt>DomainprepProfile</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return  The global <tt>DomainprepProfile</tt> instance.
     * @since   JSO-0.12.3
     */
    public static synchronized DomainprepProfile getInstance() {
        if (_Inst == null)
            _Inst = new DomainprepProfile();
        return _Inst;
    }
}