/*
 * CommandActions.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.commands;


import java.util.Set;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamElement;

/**
 * <p>
 * Defines the interface for specifying allowed and default actions for a stage
 * in command execution.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface CommandActions extends StreamElement {
    //"Constants"
    /** The qualified name for <tt>CommandActions</tt>
     * ({http://jabber.org/protocol/commands}info).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME    = new NSI("actions", CommandQuery.NAMESPACE);

    //Methods
    /**
     * <p>
     * Retrieves the set of allowed actions for this
     * <tt>CommandActions</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a <tt>CommandQuery.Action</tt> instance.</p>
     *
     * @return The current set of allowed actions.
     * @since   JSO-0.12
     */
    public Set getAllowedActions();
    /**
     * <p>
     * Sets the allowed actions for this <tt>CommandActions</tt>.</p>
     *
     * <p>
     * The value of <tt>acts</tt> may be <tt>null</tt>, in which case it is
     * treated as an empty set. Each element of <tt>acts</tt> is treated as a
     * <tt>CommandQuery.Action</tt> object if possible, or as a string
     * otherwise.</p>
     *
     * @param acts The new set of allowed actions.
     * @since   JSO-0.12
     */
    public void setAllowedActions(Set acts);
    
    /**
     * <p>
     * Determines if the given action is allowed.</p>
     *
     * <p>
     * The value of <tt>act</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param act The action to check.
     * @return <tt>true</tt> if <tt>act</tt> is allowed.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public boolean isActionAllowed(CommandQuery.Action act) throws IllegalArgumentException;
    /**
     * <p>
     * Sets the given action as allowed or disallowed.</p>
     *
     * <p>
     * The value of <tt>act</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param act The action to check.
     * @param allow The new allow flag.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public void setActionAllowed(CommandQuery.Action act, boolean allow) throws IllegalArgumentException;

    /**
     * <p>
     * Retrieves the "execute" action for this <tt>CommandActions</tt>. This
     * value is how the command responder interprets {@link
     * CommandQuery#EXECUTE} for the current stage of command execution.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * @return The action equivalent to <tt>EXECUTE</tt>.
     * @since   JSO-0.12
     */
    public CommandQuery.Action getExecuteAction();
    /**
     * <p>
     * Sets the "execute" action for this <tt>CommandActions</tt>.</p>
     *
     * <p>
     * The value of <tt>act</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param act The action equivalent to <tt>EXECUTE</tt>.
     * @since   JSO-0.12
     */
    public void setExecuteAction(CommandQuery.Action act);
}