/*
 * PrivacyList.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.core;


import java.util.List;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamElement;

/**
 * <p>
 * Interface for representing a "jabber:iq:privacy" privacy list.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface PrivacyList extends StreamElement {
    //"Constants"
    /** The qualified name for <tt>PrivacyList</tt> ({jabber:iq:privacy}list).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME        = new NSI("list", PrivacyQuery.NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the name of this <tt>PrivacyList</tt>. This method corresponds
     * to the attribute "name".</p>
     *
     * <p>
     * This method should never return <tt>null</tt> or "".</p>
     *
     * @return The privacy list name.
     * @since   JSO-0.12
     */
    public String getName();

    /**
     * <p>
     * Retrieves the list of items contained by this <tt>PrivacyList</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * of the returned list is a <tt>PrivacyItem</tt> instance.</p>
     *
     * @return The list of contained <tt>PrivacyItem</tt> instances.
     * @since   JSO-0.12
     */
    public List listItems();
    /**
     * <p>
     * Adds a new <tt>PrivacyItem</tt> with the given action. The order is also
     * set to the next highest order available.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>act</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param act The new item's action.
     * @return The new <tt>PrivacyItem</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public PrivacyItem addItem(PrivacyItem.Action act) throws IllegalArgumentException;
    /**
     * <p>
     * Adds a new <tt>PrivacyItem</tt> with the given action and order.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>act</tt> cannot be <tt>null</tt>, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * If the value of <tt>order</tt> is less than 0, then next highest order
     * determined is used.</p>
     *
     * @param act The new item's action.
     * @param order The new item's order.
     * @return The new <tt>PrivacyItem</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public PrivacyItem addItem(PrivacyItem.Action act, int order) throws IllegalArgumentException;
    /**
     * <p>
     * Removes all <tt>PrivacyItem</tt> instances from this
     * <tt>PrivacyList</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearItems();
}