/*
 * FeatureNegQuery.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.fneg;


import java.util.List;

import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.x.xdata.XDataForm;

/**
 * <p>
 * Interface for representing the feature negotiation query.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 * @see <a href="http://www.jabber.org/jeps/jep-0020.html">JEP-0020</a>
 */
public interface FeatureNegQuery extends Extension {
    //"Constants"
    /** The namespace URI for <tt>FeatureNegQuery</tt>
     * (http://jabber.org/protocol/feature-neg).</p>
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE       = "http://jabber.org/protocol/feature-neg";
    /** The qualified name for <tt>FeatureNegQuery</tt>
     * ({http://jabber.org/protocol/feature-neg}feature).</p>
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME            = new NSI("feature", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the <tt>XDataForm</tt> for this
     * <tt>FeatureNegQuery</tt>. The form contains the actual data to
     * be negotiated.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If the form
     * does not yet exist, it is created.</p>
     *
     * @return The contained <tt>XDataForm</tt>.
     * @since   JSO-0.12
     */
    public XDataForm getForm();

    /**
     * <p>
     * Retrieves the list of feature vars for this
     * <tt>FeatureNegQuery</tt>. This is the list of feature (names)
     * to be negotiated.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a feature "var", as a string.</p>
     *
     * @return The list of feature vars to negotiate.
     * @since   JSO-0.12
     */
    public List listFeatureVars();
    /**
     * <p>
     * Determines if this <tt>FeatureNegQuery</tt> has a field for the
     * given feature.</p>
     *
     * <p>
     * The value of <tt>feat</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param feat The feature var to negotiate.
     * @return <tt>true</tt> if <tt>feat</tt> exists.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public boolean hasFeature(String feat) throws IllegalArgumentException;
    /**
     * <p>
     * Adds or modifies the given feature to this
     * <tt>FeatureNegQuery</tt>. If there is already a field that
     * matches <tt>feat</tt>, it is changed to use the given list of options.
     * Otherwise a field is added, using the given list of options.</p>
     *
     * <p>
     * The value of <tt>feat</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>opts</tt> may be <tt>null</tt>, in which case it will
     * considered to be an empty list. Each element within <tt>opt</tt> is
     * treated as a string, using the <tt>toString()</tt> method.</p>
     *
     * @param feat The feature var to put.
     * @param opts The list of feature options.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public void putFeature(String feat, List opts) throws IllegalArgumentException;
    /**
     * <p>
     * Removes the given feature from this <tt>FeatureNegQuery</tt>.</p>
     *
     * <p>
     * The value of <tt>feat</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param feat The feature var to remove.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public void removeFeature(String feat) throws IllegalArgumentException;

    /**
     * <p>
     * Retrieves the list of options for a given feature. Use {@link
     * #putFeature} to change this list.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is the value of the option, as a string.</p>
     *
     * <p>
     * The value of <tt>feat</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param feat The feature var to negotiate.
     * @return The list of feature vars to negotiate.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public List getSupportedOptions(String feat) throws IllegalArgumentException;

    /**
     * <p>
     * Determines if this <tt>FeatureNegQuery</tt> has a preferred
     * option set for the given feature.</p>
     *
     * <p>
     * The value of <tt>feat</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param feat The feature var to negotiate.
     * @return <tt>true</tt> if <tt>feat</tt> exists, and has a preferred
     * option set.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public boolean hasPreferredOption(String feat) throws IllegalArgumentException;
    /**
     * <p>
     * Retrieves the preferred option for the given feature.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Use {@link
     * #hasPreferredOption} to determine if it has been set.</p>
     *
     * <p>
     * The value of <tt>feat</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param feat The feature var to negotiate.
     * @return The preferred option for <tt>feat</tt>.
     * @throws IllegalArgumentException If the parameter is invalid.
     * @since   JSO-0.12
     */
    public String getPreferredOption(String feat) throws IllegalArgumentException;
    /**
     * <p>
     * Sets or removes the preferred option for the given feature. If there is
     * no field for <tt>feat</tt> and <tt>opt</tt> is not <tt>null</tt>, one
     * is created.</p>
     *
     * <p>
     * The value of <tt>feat</tt> cannot be <tt>null</tt> or "", or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>opt</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param feat The feature var to negotiate.
     * @param opt The new preferred option.
     * @throws IllegalArgumentException If any parameters are invalid.
     * @since   JSO-0.12
     */
    public void setPreferredOption(String feat, String opt) throws IllegalArgumentException;
}