/*
 * FileTransferProfile.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.sift;


import java.util.Date;

import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.StreamElement;
import org.jabberstudio.jso.x.si.SIProfile;

/**
 * <p>
 * Defines the interface for the file transfer profile to Stream Initiation.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 */
public interface FileTransferProfile extends SIProfile {
    //Inner-classes
    /**
     * <p>
     * Defines the interface for specifying range information.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     */
    public static interface Range extends StreamElement {
        //"Constants"
        /** The qualified name for <tt>Range</tt>
         * ({http://jabber.org/protocol/si/profile/file-transfer}range).
         *
         * @since   JSO-0.12
         */
        public static final NSI NAME        = new NSI("range", NAMESPACE);
        
        //Methods
        /**
         * <p>
         * Determines if the offset is specified for this <tt>Range</tt>.</p>
         *
         * @return <tt>true</tt> if the starting offset is specified.
         * @since   JSO-0.12
         */
        public boolean hasOffset();
        /**
         * <p>
         * Retrieves the offset for this <tt>Range</tt>. This value specifies
         * the starting byte of the file transfer.</p>
         *
         * <p>
         * The value returned by this method should never be less than 0.</p>
         *
         * @return The starting byte offset.
         * @since   JSO-0.12
         */
        public long getOffset();
        /**
         * <p>
         * Sets the offset for this <tt>Range</tt>.</p>
         *
         * <p>
         * If the value of <tt>off</tt> is less than 0, any previous value
         * is removed.</p>
         *
         * @param off The new starting offset.
         * @since   JSO-0.12
         */
        public void setOffset(long off);

        /**
         * <p>
         * Determines if the length is specified for this <tt>Range</tt>.</p>
         *
         * @return <tt>true</tt> if the length is specified.
         * @since   JSO-0.12
         */
        public boolean hasLength();
        /**
         * <p>
         * Retrieves the length for this <tt>Range</tt>. This value specifies
         * the length of the file transfer, but not necessarily of the file
         * itself.</p>
         *
         * <p>
         * The value returned by this method should never be less than 0. If
         * the length is not set, this method may return the file size, as
         * returned by {@link FileTransferProfile#getSize}.</p>
         *
         * @return The transfer length.
         * @since   JSO-0.12
         */
        public long getLength();
        /**
         * <p>
         * Sets the length for this <tt>Range</tt>.</p>
         *
         * <p>
         * If the value of <tt>len</tt> is less than 0, any previous value
         * is removed.</p>
         *
         * @param len The new transfer length.
         * @since   JSO-0.12
         */
        public void setLength(long len);
    }
    
    //"Constants"
    /** The namespace URI for <tt>FileTransferProfile</tt> and related
     * interfaces (http://jabber.org/protocol/si/profile/file-transfer).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE   = "http://jabber.org/protocol/si/profile/file-transfer";
    /** The qualified name for <tt>FileTransferProfile</tt>
     * ({http://jabber.org/protocol/si/profile/file-transfer}file).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME        = new NSI("file", NAMESPACE);
    
    //Methods
    /**
     * <p>
     * Retrieves the file name for this <tt>FileTransferProfile</tt>. This
     * value is usually the name the sender has determined, without any path
     * information.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The name of the file to transfer.
     * @since   JSO-0.12
     */
    public String getName();
    /**
     * <p>
     * Sets the file name for this <tt>FileTransferProfile</tt>.</p>
     *
     * <p>
     * The value of <tt>name</tt> may be <tt>null</tt> or "", in which case
     * any previous value is removed.</p>
     *
     * @param name The new file name.
     * @since   JSO-0.12
     */
    public void setName(String name);

    /**
     * <p>
     * Retrieves the file size for this <tt>FileTransferProfile</tt>. This
     * value is the total size of the file to be transferred, but is not
     * necessarily the actual amount of data to be transferred.</p>
     *
     * <p>
     * The value returned by this method is never less than 0.</p>
     *
     * @return The size of the file to transfer.
     * @since   JSO-0.12
     */
    public long getSize();
    /**
     * <p>
     * Sets the file size for this <tt>FileTransferProfile</tt>.</p>
     *
     * <p>
     * The value of <tt>size</tt> may be less than 0, in which case any
     * previous value is removed.</p>
     *
     * @param size The new file size.
     * @since   JSO-0.12
     */
    public void setSize(long size);

    /**
     * <p>
     * Retrieves the hash for this <tt>FileTransferProfile</tt>. This value is
     * the MD5 hash of the file contents to be transferred.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The MD5 hash of the file to transfer.
     * @since   JSO-0.12
     */
    public byte[] getHash();
    /**
     * <p>
     * Sets the hash for this <tt>FileTransferProfile</tt>.</p>
     *
     * <p>
     * The value of <tt>hash</tt> may be <tt>null</tt> or or a zero-length
     * array, in which case any previous value is removed.</p>
     *
     * @param hash The new MD5 hash value.
     * @since   JSO-0.12
     */
    public void setHash(byte[] hash);
    
    /**
     * <p>
     * Retrieves the file date for this <tt>FileTransferProfile</tt>. This
     * value is usually the last-modified date of the file to be
     * transferred.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt>.</p>
     *
     * @return The last-modified date of the file to transfer.
     * @since   JSO-0.12
     */
    public Date getDate();
    /**
     * <p>
     * Sets the file date for this <tt>FileTransferProfile</tt>.</p>
     *
     * <p>
     * The value of <tt>date</tt> may be <tt>null</tt>, in which case any
     * previous value is removed.</p>
     *
     * @param date The new file date.
     * @since   JSO-0.12
     */
    public void setDate(Date date);
    
    /**
     * <p>
     * Retrieves the description for this <tt>FileTransferProfile</tt>. This
     * value is a human-readable description of the file, or the reason for
     * the transfer.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The description of the file to transfer.
     * @since   JSO-0.12
     */
    public String getDescription();
    /**
     * <p>
     * Sets the description for this <tt>FileTransferProfile</tt>.</p>
     *
     * <p>
     * The value of <tt>desc</tt> may be <tt>null</tt> or "", in which case any
     * previous value is removed.</p>
     *
     * @param desc The new description.
     * @since   JSO-0.12
     */
    public void setDescription(String desc);

    /**
     * <p>
     * Determines if a range is specified for this
     * <tt>FileTransferProfile</tt>.</p>
     *
     * @return <tt>true</tt> if a <tt>Range</tt> instance if present.
     * @since   JSO-0.12
     */
    public boolean hasRange();
    /**
     * <p>
     * Retrieves the range for this <tt>FileTransferProfile</tt>. This value
     * determines the offset and length to be transferred.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If not already
     * present, an instance of <tt>Range</tt> is first created and added.</p>
     *
     * @return The range for this <tt>FileTransferProfile</tt>.
     * @since   JSO-0.12
     */
    public Range getRange();
    /**
     * <p>
     * Sets the range for this <tt>FileTransferProfile</tt>. This method always
     * ensures a <tt>Range</tt> instance exists.  Use {@link #removeRange} to
     * remove the instance.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. If a
     * <tt>Range</tt> instance is already present, is is modified.</p>
     *
     * <p>
     * The value of <tt>off</tt> cannot be less than 0, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * <p>
     * The value of <tt>len</tt> cannot be less than 0, or an
     * <tt>IllegalArgumentException</tt> is thrown.</p>
     *
     * @param off The new range offset.
     * @param len The new range length.
     * @return The current or new <tt>Range</tt> instance.
     * @throws IllegalArgumentException If the paramters are invalid.
     * @since   JSO-0.12
     */
    public Range putRange(long off, long len) throws IllegalArgumentException;
    /**
     * <p>
     * Removes the range from this <tt>FileTransferProfile</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void removeRange();
}