/*
 * XDataForm.java
 *
 * The contents of this file are subject to the GNU Lesser General Public License
 * Version 2.1 (the "License"). You may not copy or use this file, in either
 * source code or executable form, except in compliance with the License. You
 * may obtain a copy of the License at http://www.fsf.org/licenses/lgpl.txt or
 * http://www.opensource.org/. Software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express
 * or implied. See the License for the specific language governing rights and
 * limitations under the License.
 */

package org.jabberstudio.jso.x.xdata;


import java.util.List;

import org.jabberstudio.jso.Extension;
import org.jabberstudio.jso.NSI;
import org.jabberstudio.jso.util.Enumerator;

/**
 * <p>
 * Interface for representing the root-level "jabber:x:data" form. This
 * interface maps to the &lt;x/&gt; element.</p>
 *
 * @author  linuxwolf
 * @version 0.13-DEV-20050624155226
 * @since   JSO-0.12
 * @see <a href="http://www.jabber.org/jeps/jep-0004.html">JEP-0004</a>
 */
public interface XDataForm extends XDataFieldContainer, Extension {
    //Inner-classes
    /**
     * <p>
     * Class for representing the type of <tt>XDataForm</tt>.</p>
     *
     * @author  linuxwolf
     * @version 0.13-DEV-20050624155226
     * @since   JSO-0.12
     */
    public static final class Type extends Enumerator {
        //Constructors
        private Type(String name) {
            super(name);
        }
    }
    
    //"Constants"
    /**
     * The namespace URI for <tt>XDataForm</tt> (jabber:x:data).
     *
     * @since   JSO-0.12
     */
    public static final String  NAMESPACE   = "jabber:x:data";
    /**
     * The qualified name for <tt>XDataForm</tt> ({jabber:x:data}x).
     *
     * @since   JSO-0.12
     */
    public static final NSI     NAME        = new NSI("x", NAMESPACE);
    
    /** 
     * Represents the type "form".
     *
     * @since   JSO-0.12
     */
    public static final Type    FORM        = new Type("form");
    /** 
     * Represents the type "submit".
     *
     * @since   JSO-0.12
     */
    public static final Type    SUBMIT      = new Type("submit");
    /** 
     * Represents the type "result".
     *
     * @since   JSO-0.12
     */
    public static final Type    RESULT      = new Type("result");
    /** 
     * Represents the type "cancel".
     *
     * @since   JSO-0.12
     */
    public static final Type    CANCEL      = new Type("cancel");
    
    //Methods
    /**
     * <p>
     * Retrieves the type of this <tt>XDataForm</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has not
     * been set.</p>
     *
     * @return The <tt>Type</tt> for this <tt>XDataForm</tt>.
     * @since   JSO-0.12
     */
    public Type getType();
    /**
     * <p>
     * Sets the type for this <tt>XDataForm</tt>.</p>
     *
     * <p>
     * The value of <tt>type</tt> may be <tt>null</tt>, in which case the
     * previous value is removed.</p>
     *
     * @param type The new <tt>Type</tt>.
     * @since   JSO-0.12
     */
    public void setType(Type type);

    /**
     * <p>
     * Retrieves the title for this <tt>XDataForm</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The title for this <tt>XDataForm</tt>.
     * @since   JSO-0.12
     */
    public String getTitle();
    /**
     * <p>
     * Sets the title for this <tt>XDataForm</tt>.</p>
     *
     * <p>
     * The value of <tt>title</tt> may be <tt>null</tt> or "", in which case
     * the previous value is removed.</p>
     * 
     * @param title The new title.
     * @since   JSO-0.12
     */
    public void setTitle(String title);

    /**
     * <p>
     * Retrieves the instructions for this <tt>XDataForm</tt>. This method
     * returns mult-line instructions, using the system property
     * "line.separator" to join each instruction.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The instructions for this <tt>XDataForm</tt>.
     * @since   JSO-0.12
     */
    public String getInstructions();
    /**
     * <p>
     * Sets the instructions for this <tt>XDataForm</tt>.</p>
     *
     * <p>
     * This method attempts to separate <tt>inst</tt> into multiple instruction
     * lines using the system property "line.separator" as the delimiter.</p>
     *
     * <p>
     * The value of <tt>inst</tt> may be <tt>null</tt> or "", in which case
     * any previous values are removed.</p>
     * 
     * @param inst The new instructions.
     * @since   JSO-0.12
     */
    public void setInstructions(String inst);
    /**
     * <p>
     * Retrieves the list of instructions for this <tt>XDataForm</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a String.</p>
     *
     * @return The list of instructions.
     * @since   JSO-0.12
     */
    public List listInstructions();
    /**
     * <p>
     * Adds a "line" of instructions to this <tt>XDataForm</tt>.</p>
     *
     * <p>
     * The value of <tt>inst</tt> may be <tt>null</tt> or "", in which case an
     * "empty" instruction is added.</p>
     *
     * @param inst The new instruction to add.
     * @since   JSO-0.12
     */
    public void addInstruction(String inst);
    /**
     * <p>
     * Removes the given "line" of instructions from this
     * <tt>XDataForm</tt>.</p>
     *
     * <p>
     * If <tt>inst</tt> is not found in the current instructions, this method
     * does nothing.</p>
     *
     * <p>
     * The value of <tt>inst</tt> may be <tt>null</tt> or "", this method does
     * nothing.</p>
     *
     * @param inst The instruction to remove.
     * @since   JSO-0.12
     */
    public void removeInstruction(String inst);
    /**
     * <p>
     * Removes all of the instructions from this <tt>XDataForm</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearInstructions();
    
    /**
     * <p>
     * Retrieves the reported fields section for this <tt>XDataForm</tt>.</p>
     *
     * <p>
     * The value returned by this method may be <tt>null</tt> if it has not
     * been set.</p>
     *
     * @return The reported fields section for this <tt>XDataForm</tt>.
     * @since   JSO-0.12
     */
    public XDataReported getReported();
    /**
     * <p>
     * Adds a reported fields section for this <tt>XDataForm</tt>. If this
     * section already exists, that instance is returned.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     * 
     * @return The new reported fields section.
     * @since   JSO-0.12
     */
    public XDataReported addReported();
    /**
     * <p>
     * Removes a reported fields section for this <tt>XDataForm</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void removeReported();
    
    /**
     * <p>
     * Retrieves the list of field items for this <tt>XDataForm</tt>.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>. Each element
     * in the returned list is a non-<tt>null</tt> instance of
     * <tt>XDataItem</tt>.</p>
     *
     * @return The list of contained <tt>XDataItem</tt> objects.
     * @since   JSO-0.12
     */
    public List listItems();
    /**
     * <p>
     * Adds a new field item to this <tt>XDataForm</tt>. The new
     * <tt>XDataItem</tt> is added at the end of the list.</p>
     *
     * <p>
     * The value returned by this method is never <tt>null</tt>.</p>
     *
     * @return The new field item.
     * @since   JSO-0.12
     */
    public XDataItem addItem();
    /**
     * <p>
     * Removes all field items from this <tt>XDataForm</tt>.</p>
     *
     * @since   JSO-0.12
     */
    public void clearItems();
    
    /**
     * <p>
     * Adds a new "fixed" field with the given value.</p>
     *
     * <p>
     * This method never returns <tt>null</tt>.</p>
     *
     * <p>
     * The value of <tt>val</tt> may be <tt>null</tt> or "", in which case it
     * is considered to be "".</p>
     *
     * @param val The value for the new "fixed" <tt>XDataField</tt>.
     * @return The new "fixed" <tt>XDataField</tt>.
     * @since   JSO-0.12
     */
    public XDataField addFixedField(String val);
}